/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.adapter;

import java.util.Iterator;
import java.util.Vector;

import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.DataBean.Artifact;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.DisplayBean.PFlatDisplayBean.PFlatDisplayBean;
import ca.uvic.csr.shrimp.FilterBean.FilterBean;
import ca.uvic.csr.shrimp.SearchBean.ArtifactSearchStrategy;
import ca.uvic.csr.shrimp.SearchBean.SearchBean;
import ca.uvic.csr.shrimp.SearchBean.SearchStrategy;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.gui.HierarchicalView.DisplayModeChangeListener;
import ca.uvic.csr.shrimp.gui.HierarchicalView.HierarchicalView;
import ca.uvic.csr.shrimp.gui.HierarchicalView.HvSvBridge;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;

/**
 * This adapter listens for changes in the Hierachical view's display mode (ex. "Navigation Mode")
 * 
 * @author Rob Lintern
 */
public class HvDisplayModeChangeAdapter implements DisplayModeChangeListener {

	private HierarchicalView hierarchicalView;
	private ShrimpView shrimpView;
	private ShrimpProject project;
	private static HvSearchCompletedAdapter hvsca; // accentuates search results in hierarchical view

	public HvDisplayModeChangeAdapter(ShrimpView shrimpView, HierarchicalView hierarchicalView, ShrimpProject project) {
		this.shrimpView = shrimpView;
		this.hierarchicalView = hierarchicalView;
		this.project = project;
		hvsca = new HvSearchCompletedAdapter(hierarchicalView, project);
	}

	/**
	 * @see DisplayModeChangeListener#displayModeChange(int, int)
	 */
	public void displayModeChange(int oldMode, int newMode) {
		try {
			SearchBean searchBean = (SearchBean) project.getBean(ShrimpProject.SEARCH_BEAN);
			SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
			FilterBean filterBean = (FilterBean) shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);

			SearchStrategy searchStrategy = searchBean.getStrategy(ArtifactSearchStrategy.NAME);

			// clear accentuated
			try {
				PFlatDisplayBean flatDisplayBean = (PFlatDisplayBean) hierarchicalView.getBean(ShrimpTool.DISPLAY_BEAN);
				flatDisplayBean.clearAccentuated();
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}

			// exit current mode
			switch (oldMode) {
				case HierarchicalView.NAVIGATION_MODE:
					exitNavigationMode();
					break;
				case HierarchicalView.FILTERED_MODE:
					exitFilteredMode();
					break;
				case HierarchicalView.SEARCH_MODE:
					exitSearchMode(searchStrategy);
					break;
				default: 
					// do nothing
			}
			// enter new mode
			switch (newMode) {
				case HierarchicalView.NAVIGATION_MODE:
					enterNavigationMode(selectorBean);
					break;
				case HierarchicalView.FILTERED_MODE:
					enterFilteredMode(filterBean);
					break;
				case HierarchicalView.SEARCH_MODE:
					enterSearchMode(searchStrategy);
					break;
				default: 
					// do nothing
			}

		} catch (BeanNotFoundException bnfe) {
			bnfe.printStackTrace();
		}
	}

	private void enterNavigationMode(SelectorBean selectorBean) {
		// accentuate hier view nodes that are selected in ShrimpView
		Vector svSelectedNodes = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_NODES);

		try {
			PFlatDisplayBean flatDisplayBean = (PFlatDisplayBean) hierarchicalView.getBean(ShrimpTool.DISPLAY_BEAN);
			// convert shrimp view nodes to hier nodes
			Vector hvNodes = HvSvBridge.svNodesToHvNodes(hierarchicalView, shrimpView, svSelectedNodes, false);
			flatDisplayBean.accentuate(hvNodes, true);
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

	private void exitNavigationMode() {
		// nothing
	}

	private void enterFilteredMode(FilterBean filterBean) {
		// Accentuate all filtered artifacts
		// Note: An artifact is filtered if it has at least one filter acting
		// upon it. An artifact could be filtered by type and by id but currently
		// the user has no way to tell which filters cause an artifact to be "filtered."
		try {
			PFlatDisplayBean flatDisplayBean = (PFlatDisplayBean) hierarchicalView.getBean(ShrimpTool.DISPLAY_BEAN);
			Vector nodes = flatDisplayBean.getAllNodes();
			for (Iterator iter = nodes.iterator(); iter.hasNext();) {
				ShrimpNode hierNode = (ShrimpNode) iter.next();
				if (filterBean.isFiltered(hierNode.getArtifact())) {
					flatDisplayBean.accentuate(hierNode, true);
				}
			}
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

	private void exitFilteredMode() {
		// nothing
	}

	private void enterSearchMode(SearchStrategy searchStrategy) {
		try {
			Vector svSearchResults = searchStrategy.getSearchResults();

			PFlatDisplayBean flatDisplayBean = (PFlatDisplayBean) hierarchicalView.getBean(ShrimpTool.DISPLAY_BEAN);
			for (Iterator iter = svSearchResults.iterator(); iter.hasNext();) {
				Object svObject = iter.next();
				if (svObject instanceof Artifact) {
					Vector hierNodes = flatDisplayBean.getDataDisplayBridge().getShrimpNodes((Artifact) svObject, false);
					flatDisplayBean.accentuate(hierNodes, true);
				}
			}
			searchStrategy.addSearchCompletedListener(hvsca);

		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

	private void exitSearchMode(SearchStrategy searchStrategy) {
		searchStrategy.removeSearchCompletedListener(hvsca);
	}

}
